/*
 * Copyright (C) 2018 Jean-Rémy Buchs <jrb0001@692b8c32.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.as205315.applications.ora.server.authentication;

import io.vertx.core.Vertx;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.function.Supplier;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import net.as205315.applications.ora.server.commands.CommandHandler;
import net.as205315.applications.ora.server.lobby.ClientInfo;
import net.as205315.applications.ora.server.lobby.GlobalSettings;

/**
 *
 * @author Jean-Rémy Buchs <jrb0001@692b8c32.de>
 */
public class AuthenticationManager {

    private final List<Authenticator> authenticators;

    public AuthenticationManager(GlobalSettings globalSettings, Vertx vertx, Supplier<Stream<ClientInfo>> clients) {
        authenticators = Arrays.asList(
                new ServerPasswordAuthenticator(globalSettings),
                new JwtAuthenticator(globalSettings, vertx),
                new TemporaryGrantAuthenticator(clients),
                new DynamicAdminAuthenticator(clients)
        );
    }

    public ClientAuthentication authenticate(String username, String password, String remote, ClientInfo clientInfo) {
        return new ClientAuthentication(remote, authenticators.stream()
                .map(a -> a.authenticate(username, password, remote, clientInfo))
                .filter(Objects::nonNull)
                .collect(Collectors.toList()));
    }

    public Stream<CommandHandler> getCommandHandlers() {
        return authenticators.stream()
                .filter(a -> a instanceof CommandHandler)
                .map(a -> (CommandHandler) a);
    }
}
