/*
 * Copyright (C) 2018 Jean-Rémy Buchs <jrb0001@692b8c32.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.as205315.applications.ora.server.commands;

import java.util.List;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.function.Supplier;
import java.util.stream.Stream;
import net.as205315.applications.ora.server.authentication.ClientAuthentication;
import net.as205315.applications.ora.server.lobby.Client;
import net.as205315.applications.ora.server.packets.AuthenticationErrorOrder;
import net.as205315.applications.ora.server.packets.MessageOrder;

/**
 *
 * @author Jean-Rémy Buchs <jrb0001@692b8c32.de>
 */
public class AuthenticationCommandHandler implements CommandHandler {

    private final Supplier<Stream<Client>> clients;

    public AuthenticationCommandHandler(Supplier<Stream<Client>> clients) {
        this.clients = clients;
    }

    @Override
    public boolean runCommand(Client client, String command, List<String> args) {
        switch (command) {
            case "auth":
                if (args.size() <= 1) {
                    Stream<Client> targets;
                    if (args.isEmpty()) {
                        targets = Stream.of(client);
                    } else {
                        if (client.getClientInfo().getClientAuthentication().requirePermissionOrInform(client, ClientAuthentication.Permission.ADMIN)) {
                            targets = clients.get().filter(c -> c.getClientInfo() != null).filter(c -> Objects.equals(args.get(0), c.getClientInfo().getName()));
                        } else {
                            return true;
                        }
                    }

                    AtomicBoolean found = new AtomicBoolean();
                    targets.forEach(c -> {
                        found.set(true);
                        c.sendPacket(new AuthenticationErrorOrder(0, 0, "Authentication requested by " + client.getClientInfo().getName() + "."));
                        c.disconnect();
                    });

                    if (!found.get()) {
                        client.sendPacket(new MessageOrder("Unknown user \"" + args.get(0) + "\""));
                    }

                    return true;
                } else {
                    return false;
                }
            default:
                return false;
        }
    }
}
