/*
 * Copyright (C) 2018 Jean-Rémy Buchs <jrb0001@692b8c32.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.as205315.applications.ora.server.commands;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;
import java.util.function.Supplier;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import net.as205315.applications.ora.server.authentication.AuthenticationManager;
import net.as205315.applications.ora.server.lobby.Client;
import net.as205315.applications.ora.server.lobby.GlobalSettings;
import net.as205315.applications.ora.server.packets.ChatOrder;
import net.as205315.applications.ora.server.packets.MessageOrder;

/**
 *
 * @author Jean-Rémy Buchs <jrb0001@692b8c32.de>
 */
public class CommandManager {

    private final List<CommandHandler> handlers;

    public CommandManager(Supplier<Stream<Client>> clients, AuthenticationManager authenticationManager, GlobalSettings settings) {
        handlers = Stream.of(
                Stream.of(
                        new AuthenticationCommandHandler(clients),
                        new KickCommandHandler(clients)
                ),
                authenticationManager.getCommandHandlers()
        ).flatMap(Function.identity()).collect(Collectors.toList());

    }

    public boolean handleMessage(Client client, ChatOrder order) {
        if (!order.getMessage().startsWith("/") && !order.getMessage().startsWith("!")) {
            return false;
        } else {
            List<String> args = new ArrayList<>();
            Matcher m = Pattern.compile("([^\"]\\S*|\"[^\"]*\")\\s*").matcher(order.getMessage().substring(1));
            while (m.find()) {
                args.add(m.group(1).replaceAll("\"", ""));
            }
            String command = args.remove(0);

            for (CommandHandler handler : handlers) {
                if (handler.runCommand(client, command, args)) {
                    return true;
                }
            }

            client.sendPacket(new MessageOrder("Unknown command: \"" + command + "\" " + args.stream().collect(Collectors.joining("\" \"", "\"", "\""))));
            return true;
        }
    }
}
