/*
 * Copyright (C) 2018 Jean-Rémy Buchs <jrb0001@692b8c32.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.as205315.applications.ora.server.lobby;

import java.net.InetSocketAddress;
import java.time.Instant;
import net.as205315.applications.ora.server.packets.Packet;
import net.as205315.applications.ora.server.proxy.LobbyServerPacket;

/**
 *
 * @author Jean-Rémy Buchs <jrb0001@692b8c32.de>
 */
public class Client {

    private final ServerState serverState;
    private final OraProxy proxy;
    private final int id;
    private final InetSocketAddress remoteAddress;
    private ClientInfo clientInfo;
    private int lastFrame;
    private Instant lastPacket;

    public Client(ServerState serverState, OraProxy proxy, int id, InetSocketAddress remoteAddress) {
        this.serverState = serverState;
        this.proxy = proxy;
        this.id = id;
        this.remoteAddress = remoteAddress;
    }

    public void sendPacket(Packet packet) {
        proxy.write(new LobbyServerPacket.ClientPrivatePacket(serverState.getId(), getId(), packet.toBuffer()));
    }

    public void disconnect() {
        proxy.write(new LobbyServerPacket.ClientForceDisconnectPacket(serverState.getId(), getId()));
    }

    public void setAllowDirect(boolean value) {
        proxy.write(new LobbyServerPacket.ClientAllowDirectPacket(serverState.getId(), getId(), value));
    }

    public int getId() {
        return id;
    }

    public ServerState getServerState() {
        return serverState;
    }

    public OraProxy getProxy() {
        return proxy;
    }

    public InetSocketAddress getRemoteAddress() {
        return remoteAddress;
    }

    public ClientInfo getClientInfo() {
        return clientInfo;
    }

    public void setClientInfo(ClientInfo clientInfo) {
        this.clientInfo = clientInfo;
    }

    public int getLastFrame() {
        return lastFrame;
    }

    public void setLastFrame(int lastFrame) {
        this.lastFrame = lastFrame;
    }

    public Instant getLastPacket() {
        return lastPacket;
    }

    public void setLastPacket(Instant lastPacket) {
        this.lastPacket = lastPacket;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 59 * hash + this.id;
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Client other = (Client) obj;
        if (this.id != other.id) {
            return false;
        }
        return true;
    }
}
