/*
 * Copyright (C) 2018 Jean-Rémy Buchs <jrb0001@692b8c32.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.as205315.applications.ora.server.lobby;

import io.vertx.core.json.JsonObject;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 *
 * @author Jean-Rémy Buchs <jrb0001@692b8c32.de>
 */
public class MapInfo {

    private final String id;
    private final String title;
    private final List<Slot> slots;

    public MapInfo(String id, JsonObject obj) {
        this.id = id;
        this.title = obj.getString("Title");
        this.slots = Collections.unmodifiableList(obj.getJsonObject("Players").stream().map(e -> new Slot((JsonObject) e.getValue())).collect(Collectors.toList()));
    }

    public String getId() {
        return id;
    }

    public String getTitle() {
        return title;
    }

    public List<Slot> getSlots() {
        return slots;
    }

    public class Slot {

        private final String id;
        private final boolean playable;
        private boolean closed;
        private final String bot;
        private final boolean allowBots;
        private final String faction;
        private final boolean lockFaction;
        private final String color;
        private final boolean lockColor;
        private final int team;
        private final boolean lockTeam;
        private final int spawn;
        private final boolean lockSpawn;
        private final boolean required;

        public Slot(JsonObject obj) {
            this.id = obj.getString("Name");
            this.playable = Boolean.parseBoolean(obj.getString("Playable", "False"));
            this.closed = Boolean.parseBoolean(obj.getString("Closed", "False"));
            this.bot = obj.getString("Bot");
            this.allowBots = Boolean.parseBoolean(obj.getString("AllowBots", "True"));
            this.faction = obj.getString("Faction");
            this.lockFaction = Boolean.parseBoolean(obj.getString("LockFaction", "False"));
            this.color = obj.getString("Color");
            this.lockColor = Boolean.parseBoolean(obj.getString("LockColor", "False"));
            this.team = Integer.parseInt(obj.getString("Team", "0"));
            this.lockTeam = Boolean.parseBoolean(obj.getString("LockTeam", "False"));
            this.spawn = Integer.parseInt(obj.getString("Spawn", "0"));
            this.lockSpawn = Boolean.parseBoolean(obj.getString("LockSpawn", "False"));
            this.required = Boolean.parseBoolean(obj.getString("Required", "False"));
        }

        public String getId() {
            return id;
        }

        public boolean isPlayable() {
            return playable;
        }

        public void setClosed(boolean closed) {
            this.closed = closed;
        }

        public boolean isClosed() {
            return closed;
        }

        public String getBot() {
            return bot;
        }

        public boolean isAllowBots() {
            return allowBots;
        }

        public String getFaction() {
            return faction;
        }

        public boolean isLockFaction() {
            return lockFaction;
        }

        public String getColor() {
            return color;
        }

        public boolean isLockColor() {
            return lockColor;
        }

        public int getTeam() {
            return team;
        }

        public boolean isLockTeam() {
            return lockTeam;
        }

        public int getSpawn() {
            return spawn;
        }

        public boolean isLockSpawn() {
            return lockSpawn;
        }

        public boolean isRequired() {
            return required;
        }
    }
}
