/*
 * Copyright (C) 2018 Jean-Rémy Buchs <jrb0001@692b8c32.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.as205315.applications.ora.server.lobby;

import io.vertx.core.net.NetSocket;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.function.Consumer;
import java.util.stream.Stream;
import net.as205315.applications.ora.server.proxy.LobbyServerPacket;
import net.as205315.applications.ora.server.utils.PacketUtils;

/**
 *
 * @author Jean-Rémy Buchs <jrb0001@692b8c32.de>
 */
public class OraProxy {

    private final NetSocket socket;
    private final int idBase;
    private final int idLimit;
    private final List<Client> clients = new ArrayList<>();
    private final Consumer<LobbyServerPacket> sender;
    private LobbyServerPacket.State state;

    public OraProxy(NetSocket socket, int idBase, int idLimit) {
        this.socket = Objects.requireNonNull(socket, "socket");
        this.idBase = idBase;
        this.idLimit = idLimit;
        sender = PacketUtils.createWriter(socket::write, LobbyServerPacket::toBuffer, false, 0);
    }

    public NetSocket getSocket() {
        return socket;
    }

    public int getIdBase() {
        return idBase;
    }

    public int getIdLimit() {
        return idLimit;
    }

    public Stream<Client> getClients() {
        return clients.stream();
    }

    public Client addClient(Client client) {
        clients.add(client);
        return client;
    }

    public void removeClient(Client client) {
        clients.remove(client);
    }

    public LobbyServerPacket.State getState() {
        return state;
    }

    public void setState(LobbyServerPacket.State state) {
        this.state = state;
    }

    public void write(LobbyServerPacket packet) {
        sender.accept(packet);
    }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = 97 * hash + Objects.hashCode(this.socket);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final OraProxy other = (OraProxy) obj;
        return Objects.equals(this.socket.remoteAddress(), other.socket.remoteAddress());
    }
}
