/*
 * Copyright (C) 2018 Jean-Rémy Buchs <jrb0001@692b8c32.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.as205315.applications.ora.server.lobby;

import io.vertx.core.Vertx;
import io.vertx.core.buffer.Buffer;
import java.io.File;
import java.time.Instant;
import java.util.logging.Level;
import java.util.logging.Logger;
import net.as205315.applications.ora.server.Config;

/**
 *
 * @author Jean-Rémy Buchs <jrb0001@692b8c32.de>
 */
public class ReplayManager {

    private static final Logger LOGGER = Logger.getLogger(ReplayManager.class.getName());

    private final Vertx vertx;

    public ReplayManager(Vertx vertx) {
        this.vertx = vertx;
    }

    public void saveReplay(GlobalSettings settings, Buffer replay) {
        File file = new File(new File(System.getProperty(Config.CONFIG_REPLAY_DIR, System.getenv(Config.CONFIG_REPLAY_DIR)), settings.getModVersion()), Instant.now().toString() + ".orarep");
        file.getParentFile().mkdirs();
        vertx.fileSystem().mkdirs(file.getParentFile().getAbsolutePath(), mkdirs -> {
            if (mkdirs.failed()) {
                LOGGER.log(Level.SEVERE, "Failed to create directories for saving replay to " + file.getAbsolutePath() + ".", mkdirs.cause());
            } else {
                vertx.fileSystem().writeFile(file.getAbsolutePath(), replay, write -> {
                    if (write.failed()) {
                        LOGGER.log(Level.SEVERE, "Failed to write replay file " + file.getAbsolutePath() + ".", mkdirs.cause());
                    } else {
                        LOGGER.log(Level.INFO, "Saved replay successfully to {0}.", file.getAbsolutePath());
                    }
                });
            }
        });
    }
}
