/*
 * Copyright (C) 2018 Jean-Rémy Buchs <jrb0001@692b8c32.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.as205315.applications.ora.server.packets;

import io.vertx.core.json.JsonObject;
import net.as205315.applications.ora.server.lobby.ClientInfo;

/**
 *
 * @author Jean-Rémy Buchs <jrb0001@692b8c32.de>
 */
public class HandshakeResponseOrder extends ServerOrder {

    private final String mod;
    private final String modVersion;
    private final String password;
    private final ClientInfo clientInfo;

    public HandshakeResponseOrder(int client, int frame, JsonObject yaml) {
        super(client, frame);

        JsonObject handshake = yaml.getJsonObject("Handshake");
        this.mod = handshake.getString("Mod");
        this.modVersion = handshake.getString("Version");
        this.password = handshake.getString("Password");
        this.clientInfo = new ClientInfo(yaml.getJsonObject("Client"));
    }

    public String getMod() {
        return mod;
    }

    public String getModVersion() {
        return modVersion;
    }

    public String getPassword() {
        return password;
    }

    public ClientInfo getClientInfo() {
        return clientInfo;
    }

    @Override
    public String getOrderName() {
        return "HandshakeResponse";
    }

    @Override
    public JsonObject serialize() {
        throw new UnsupportedOperationException(getClass().getName() + " shouldn't be sent!");
    }
}
