/*
 * Copyright (C) 2018 Jean-Rémy Buchs <jrb0001@692b8c32.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.as205315.applications.ora.server.packets;

import io.vertx.core.buffer.Buffer;
import io.vertx.core.json.JsonObject;
import java.time.Instant;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import net.as205315.applications.ora.server.lobby.GlobalSettings;
import net.as205315.applications.ora.server.utils.MiniYamlParser;

/**
 *
 * @author Jean-Rémy Buchs <jrb0001@692b8c32.de>
 */
public class ReplayMetadataOrder extends Packet {

    private static final int START_MARKER = -1;
    private static final int END_MARKER = -2;
    private static final int VERSION = 0x00000001;
    private static final DateTimeFormatter INSTANT_FORMATTER = DateTimeFormatter.ofPattern("yyyy-MM-dd HH-mm-ss").withZone(ZoneOffset.UTC);

    private final GlobalSettings settings;
    private final Instant start;
    private final Instant end;

    public ReplayMetadataOrder(int client, int frame, GlobalSettings settings, Instant start, Instant end) {
        super(client, frame);

        this.settings = settings;
        this.start = start;
        this.end = end;
    }

    @Override
    public Buffer toBuffer() {
        throw new UnsupportedOperationException("This packet should never be sent over a regular connection.");
    }

    @Override
    public Buffer toReplayBuffer() {
        Buffer data = Buffer.buffer(MiniYamlParser.print(serialize()));

        Buffer buffer = Buffer.buffer();
        buffer.appendIntLE(START_MARKER);
        buffer.appendIntLE(VERSION);
        buffer.appendIntLE(data.length());
        buffer.appendBuffer(data);
        buffer.appendIntLE(data.length() + 4);
        buffer.appendIntLE(END_MARKER);
        return buffer;
    }

    public JsonObject serialize() {
        JsonObject root = new JsonObject();
        root.put("Mod", settings.getMod());
        root.put("Version", settings.getModVersion());
        root.put("MapUid", settings.getMap().getId());
        root.put("MapTitle", settings.getMap().getTitle());
        root.put("StartTimeUtc", INSTANT_FORMATTER.format(start));
        root.put("EndTimeUtc", INSTANT_FORMATTER.format(end));

        JsonObject obj = new JsonObject();
        obj.put("Root", root);
        return obj;
    }
}
