/*
 * Copyright (C) 2018 Jean-Rémy Buchs <jrb0001@692b8c32.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.as205315.applications.ora.server.packets;

import io.vertx.core.buffer.Buffer;
import io.vertx.core.json.JsonObject;
import net.as205315.applications.ora.server.utils.CSCompat;
import net.as205315.applications.ora.server.utils.MiniYamlParser;

/**
 *
 * @author Jean-Rémy Buchs <jrb0001@692b8c32.de>
 */
public abstract class ServerOrder extends Packet {

    public ServerOrder(int client, int frame) {
        super(client, frame);
    }

    @Override
    public Buffer toBuffer() {
        Buffer buffer = Buffer.buffer();
        buffer.appendUnsignedIntLE(getClient());
        buffer.appendUnsignedIntLE(getFrame());
        buffer.appendUnsignedByte((short) 0xFE);
        Buffer orderName = Buffer.buffer(getOrderName());
        CSCompat.append7BitInt(buffer, orderName.length());
        buffer.appendBuffer(orderName);
        Buffer data = Buffer.buffer(MiniYamlParser.print(serialize()));
        CSCompat.append7BitInt(buffer, data.length());
        buffer.appendBuffer(data);
        return buffer;
    }

    @Override
    public Buffer toReplayBuffer() {
        Buffer buffer = Buffer.buffer();
        buffer.appendUnsignedIntLE(getClient());
        buffer.appendUnsignedIntLE(0);
        buffer.appendUnsignedIntLE(getFrame());
        buffer.appendUnsignedByte((short) 0xFE);
        Buffer orderName = Buffer.buffer(getOrderName());
        CSCompat.append7BitInt(buffer, orderName.length());
        buffer.appendBuffer(orderName);
        Buffer data = Buffer.buffer(MiniYamlParser.print(serialize()));
        CSCompat.append7BitInt(buffer, data.length());
        buffer.appendBuffer(data);

        // Set size.
        buffer.setIntLE(4, buffer.length() - 8);

        return buffer;
    }

    public abstract String getOrderName();

    public abstract JsonObject serialize();
}
