/*
 * Copyright (C) 2018 Jean-Rémy Buchs <jrb0001@692b8c32.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.as205315.applications.ora.server.packets;

import io.vertx.core.json.JsonObject;
import java.util.List;
import java.util.LongSummaryStatistics;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import net.as205315.applications.ora.server.lobby.ClientInfo;

/**
 *
 * @author Jean-Rémy Buchs <jrb0001@692b8c32.de>
 */
public class SyncClientPingsOrder extends ServerOrder {

    private final List<ClientInfo> clients;

    public SyncClientPingsOrder(int client, int frame, Stream<ClientInfo> clients) {
        super(client, frame);

        this.clients = clients.collect(Collectors.toList());
    }

    @Override
    public String getOrderName() {
        return "SyncClientPings";
    }

    @Override
    public JsonObject serialize() {
        JsonObject obj = new JsonObject();

        clients.forEach((c) -> {
            List<Long> history = c.getLatencyHistory();
            LongSummaryStatistics statistics = history.stream().mapToLong(Long::longValue).summaryStatistics();
            JsonObject ping = new JsonObject();
            ping.put("Index", c.getId());
            ping.put("Latency", Long.toString((long) (statistics.getAverage() / 1000 / 1000)));
            ping.put("LatencyJitter", Long.toString((statistics.getMax() - statistics.getMin()) / 2 / 1000 / 1000));
            ping.put("LatencyHistory", history.stream().mapToLong(Long::longValue).map(value -> value / 1000 / 1000).mapToObj(String::valueOf).collect(Collectors.joining(", ")));
            obj.put("ClientPing@" + c.getId(), ping);
        });

        return obj;
    }
}
