/*
 * Copyright (C) 2018 Jean-Rémy Buchs <jrb0001@692b8c32.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.as205315.applications.ora.server.packets;

import io.vertx.core.json.JsonObject;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import net.as205315.applications.ora.server.lobby.ClientInfo;
import net.as205315.applications.ora.server.lobby.GlobalSettings;
import net.as205315.applications.ora.server.lobby.MapInfo;

/**
 *
 * @author Jean-Rémy Buchs <jrb0001@692b8c32.de>
 */
public class SyncInfoOrder extends ServerOrder {

    private final GlobalSettings settings;
    private final List<ClientInfo> clients;

    public SyncInfoOrder(int client, int frame, GlobalSettings settings, Stream<ClientInfo> clients) {
        super(client, frame);

        this.settings = settings;
        this.clients = clients.collect(Collectors.toList());
    }

    @Override
    public String getOrderName() {
        return "SyncInfo";
    }

    @Override
    public JsonObject serialize() {
        JsonObject obj = new JsonObject();

        obj.put("GlobalSettings", settings.serialize());
        if (settings.getMap() != null) {
            settings.getMap().getSlots().stream().filter(MapInfo.Slot::isPlayable).forEach((s) -> {
                JsonObject sobj = new JsonObject();
                sobj.put("PlayerReference", s.getId());
                sobj.put("Playable", s.isPlayable() ? "True" : "False");
                sobj.put("Closed", s.isClosed() ? "True" : "False");
                sobj.put("Bot", s.getBot());
                sobj.put("AllowBots", s.isAllowBots() ? "True" : "False");
                sobj.put("Faction", s.getFaction());
                sobj.put("LockFaction", s.isLockFaction() ? "True" : "False");
                sobj.put("LockColor", s.isLockColor() ? "True" : "False");
                sobj.put("Team", s.getTeam());
                sobj.put("LockTeam", s.isLockTeam() ? "True" : "False");
                sobj.put("Spawn", s.getSpawn());
                sobj.put("LockSpawn", s.isLockSpawn() ? "True" : "False");
                sobj.put("Required", s.isRequired() ? "True" : "False");
                obj.put("Slot@" + s.getId(), sobj);
            });
        }
        clients.forEach((c) -> {
            obj.put("Client@" + c.getId(), c.serialize());
        });

        return obj;
    }
}
