/*
 * Copyright (C) 2018 Jean-Rémy Buchs <jrb0001@692b8c32.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.as205315.applications.ora.server.proxy;

import io.vertx.core.Vertx;
import io.vertx.core.net.NetSocket;
import io.vertx.core.net.SocketAddress;
import java.text.MessageFormat;
import java.util.function.Consumer;
import java.util.logging.Level;
import java.util.logging.Logger;
import net.as205315.applications.ora.server.utils.PacketUtils;

/**
 *
 * @author Jean-Rémy Buchs <jrb0001@692b8c32.de>
 */
public class LobbyServerConnection {

    private static final Logger LOG = Logger.getLogger(LobbyServerConnection.class.getName());

    private final String lobbyAddress;
    private final int lobbyPort;
    private final Consumer<LobbyServerPacket> packetHandler;
    private final Vertx vertx;
    private NetSocket lobbyServerConnection;
    private Consumer<LobbyServerPacket> sender;

    public LobbyServerConnection(String lobbyAddress, int lobbyPort, Consumer<LobbyServerPacket> packetHandler, Vertx vertx) {
        this.lobbyAddress = lobbyAddress;
        this.lobbyPort = lobbyPort;
        this.packetHandler = packetHandler;
        this.vertx = vertx;

        connect(null);
    }

    private void connect(Object v) {
        LOG.log(Level.INFO, "Connecting to lobby server: {0}:{1}.", new Object[]{lobbyAddress, lobbyPort});
        vertx.createNetClient().connect(SocketAddress.inetSocketAddress(lobbyPort, lobbyAddress), cres -> {
            if (cres.succeeded()) {
                LOG.log(Level.INFO, "Connected to lobby server: {0}:{1}.", new Object[]{lobbyAddress, lobbyPort});

                lobbyServerConnection = cres.result();
                lobbyServerConnection.closeHandler(this::connect);
                lobbyServerConnection.handler(PacketUtils.createReader(LobbyServerPacket::fromBuffer, packetHandler, false, 0));
                sender = PacketUtils.createWriter(lobbyServerConnection::write, LobbyServerPacket::toBuffer, false, 0);
            } else {
                LOG.log(Level.SEVERE, MessageFormat.format("Failed to connect to lobby server: {0}:{1}.", new Object[]{lobbyAddress, lobbyPort}), cres.cause());
                vertx.setTimer(1000, this::connect);
            }
        });
    }

    public void send(LobbyServerPacket packet) {
        sender.accept(packet);
    }

    public void close() {
        lobbyServerConnection.close();
    }
}
