/*
 * Copyright (C) 2018 Jean-Rémy Buchs <jrb0001@692b8c32.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.as205315.applications.ora.server.proxy;

import io.vertx.core.buffer.Buffer;

/**
 *
 * @author Jean-Rémy Buchs <jrb0001@692b8c32.de>
 */
public class ProxyServerPacket {

    private static final int HEADER_SIZE = 4;

    private final int lobby;
    protected final Buffer payloadBuffer;

    public ProxyServerPacket(int lobby, Buffer payloadBuffer) {
        this.lobby = lobby;
        this.payloadBuffer = payloadBuffer;
    }

    public int getLobby() {
        return lobby;
    }

    public Buffer getPayloadBuffer() {
        return payloadBuffer;
    }

    public Buffer toBuffer() {
        Buffer buffer = Buffer.buffer(HEADER_SIZE + payloadBuffer.length());

        buffer.appendInt(lobby);
        buffer.appendBuffer(payloadBuffer);

        return buffer;
    }

    public static ProxyServerPacket fromBuffer(Buffer buffer) {
        return new ProxyServerPacket(buffer.getInt(0), buffer.slice(HEADER_SIZE, buffer.length()));
    }
}
