/*
 * Copyright (C) 2018 Jean-Rémy Buchs <jrb0001@692b8c32.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.as205315.applications.ora.server.utils;

import io.vertx.core.buffer.Buffer;

/**
 *
 * @author Jean-Rémy Buchs <jrb0001@692b8c32.de>
 */
public class CSCompat {

    public static int read7BitIntLength(Buffer buf, int pos) {
        int length = 0;
        for (int i = 0; i < 5; i++) {
            int part = buf.getUnsignedByte(pos + i);

            length++;
            if ((part & 0x80) != 0x80) {
                break;
            }
        }
        return length;
    }

    public static int read7BitInt(Buffer buf, int pos) {
        int result = 0;
        for (int i = 0; i < 5; i++) {
            int part = buf.getUnsignedByte(pos + i);
            result |= (part & 0x7F) << (7 * i);
            if ((part & 0x80) != 0x80) {
                break;
            }
        }
        return result;
    }

    public static int write7BitInt(Buffer buf, int pos, int value) {
        int length = 0;
        boolean abort = false;
        for (int i = 0; i < 5; i++) {
            if ((value >>> 7) == 0) {
                abort = true;
            }
            buf.setByte(pos + i, (byte) (abort ? value : value | 0x80));
            value >>>= 7;
            length++;
            if (abort) {
                break;
            }
        }
        return length;
    }

    public static int append7BitInt(Buffer buf, int value) {
        int length = 0;
        boolean abort = false;
        for (int i = 0; i < 5; i++) {
            if ((value >>> 7) == 0) {
                abort = true;
            }
            buf.appendByte((byte) (abort ? value : value | 0x80));
            value >>>= 7;
            length++;
            if (abort) {
                break;
            }
        }

        return length;
    }
}
