/*
 * Copyright (C) 2018 Jean-Rémy Buchs <jrb0001@692b8c32.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.as205315.applications.ora.server.utils;

import io.vertx.core.json.JsonObject;
import java.util.Arrays;
import java.util.Collections;
import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;

/**
 *
 * @author Jean-Rémy Buchs <jrb0001@692b8c32.de>
 */
public class MiniYamlParser {

    public static JsonObject parse(String yaml) {
        yaml = yaml.replace("    ", "\t");

        return parse(new LinkedList<>(Arrays.asList(yaml.split("\n"))), 0);
    }

    private static JsonObject parse(Queue<String> lines, int indentation) {
        String prefix = String.join("", Collections.nCopies(indentation, "\t"));
        JsonObject obj = new JsonObject();

        while (lines.peek() != null && lines.peek().startsWith(prefix)) {
            String[] parts = lines.poll().trim().replaceAll("#.*$", "").split(":", 2);
            if (parts[0].isEmpty()) {
                // Ignore empty lines.
            } else if ((lines.peek() != null) && (lines.peek().trim().length() >= indentation + 1) && (lines.peek().charAt(indentation) == '\t')) {
                JsonObject child = parse(lines, indentation + 1);
                if (!parts[1].trim().isEmpty()) {
                    child.put("_", parts.length == 1 ? null : parts[1].trim());
                }
                obj.put(parts[0].trim(), child);
            } else {
                obj.put(parts[0].trim(), parts.length == 1 ? null : parts[1].trim().replace("\\n", "\n"));
            }
        }

        return obj;
    }

    public static String print(JsonObject obj) {
        if (obj.containsKey("_")) {
            return String.valueOf(obj.getValue("_"));
        }

        StringBuilder yaml = new StringBuilder();

        print(obj, 0, yaml);

        return yaml.toString();
    }

    private static void print(JsonObject obj, int indentation, StringBuilder yaml) {
        String prefix = String.join("", Collections.nCopies(indentation, "\t"));
        for (Map.Entry<String, Object> e : obj) {
            if ("_".equals(e.getKey())) {
                continue;
            }

            yaml.append(prefix);
            yaml.append(e.getKey());
            yaml.append(':');
            if (e.getValue() instanceof JsonObject) {
                JsonObject child = (JsonObject) e.getValue();
                if (child.containsKey("_")) {
                    yaml.append(' ');
                    yaml.append(child.getValue("_"));
                }
                yaml.append('\n');
                print(child, indentation + 1, yaml);
            } else {
                yaml.append(' ');
                yaml.append(e.getValue() == null ? "" : e.getValue());
                yaml.append('\n');
            }
        }
    }
}
