/*
 * Copyright (C) 2018 Jean-Rémy Buchs <jrb0001@692b8c32.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.as205315.applications.ora.server.utils;

import io.vertx.core.Handler;
import io.vertx.core.buffer.Buffer;
import java.util.function.Consumer;
import java.util.function.Function;

/**
 *
 * @author Jean-Rémy Buchs <jrb0001@692b8c32.de>
 */
public class PacketUtils {

    public static <T> Handler<Buffer> createReader(Function<Buffer, T> mapper, Consumer<T> consumer, boolean le, int offset) {
        return new ReadHandler<>(le, offset, mapper, consumer);
    }

    public static <T> Function<Consumer<Buffer>, Consumer<T>> createWriterMapper(Function<T, Buffer> mapper, boolean le, int offset) {
        return sender -> createWriter(sender, mapper, le, offset);
    }

    public static <T> Consumer<T> createWriter(Consumer<Buffer> sender, Function<T, Buffer> mapper, boolean le, int offset) {
        return packet -> {
            Buffer buffer = mapper.apply(packet);
            if (le) {
                sender.accept(Buffer.buffer(4 + buffer.length()).appendIntLE(buffer.length() + offset).appendBuffer(buffer));
            } else {
                sender.accept(Buffer.buffer(4 + buffer.length()).appendInt(buffer.length() + offset).appendBuffer(buffer));
            }
        };
    }

    private static class ReadHandler<T> implements Handler<Buffer> {

        private final boolean le;
        private final int offset;
        private final Function<Buffer, T> mapper;
        private final Consumer<T> consumer;
        private int receiveLength;
        private Buffer receiveBuffer = Buffer.buffer();

        public ReadHandler(boolean le, int offset, Function<Buffer, T> mapper, Consumer<T> consumer) {
            this.le = le;
            this.offset = offset;
            this.mapper = mapper;
            this.consumer = consumer;
        }

        @Override
        public void handle(Buffer buffer) {
            receiveBuffer.appendBuffer(buffer);
            while (receiveBuffer.length() >= (receiveLength == 0 ? 4 : receiveLength)) {
                if (receiveLength == 0) {
                    receiveLength = (le ? receiveBuffer.getIntLE(0) : receiveBuffer.getInt(0)) - offset;
                    receiveBuffer = Buffer.buffer().appendBuffer(receiveBuffer, 4, receiveBuffer.length() - 4);
                } else {
                    consumer.accept(mapper.apply(receiveBuffer.slice(0, receiveLength)));
                    receiveBuffer = Buffer.buffer().appendBuffer(receiveBuffer, receiveLength, receiveBuffer.length() - receiveLength);
                    receiveLength = 0;
                }
            }
        }
    }
}
