/*
 * Copyright (C) 2018 Jean-Rémy Buchs <jrb0001@692b8c32.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.as205315.applications.ora.server.utils;

import io.vertx.core.net.SocketAddress;
import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;

/**
 *
 * @author Jean-Rémy Buchs <jrb0001@692b8c32.de>
 */
public class SocketAddressParser {

    public static String toString(SocketAddress socketAddress) {
        return SocketAddressParser.toString(socketAddress.host(), socketAddress.port());
    }

    public static String toString(String address, int port) {
        try {
            return toString(InetAddress.getByName(address), port);
        } catch (UnknownHostException ex) {
            throw new IllegalArgumentException("Failed to parse address: " + address + ".", ex);
        }
    }

    public static String toString(InetAddress address, int port) {
        if (address instanceof Inet4Address) {
            return address.getHostAddress() + ":" + port;
        } else {
            return "[" + address.getHostAddress() + "]:" + port;
        }
    }

    public static InetSocketAddress fromString(String text) {
        return new InetSocketAddress(text.substring(0, text.lastIndexOf(":")), Integer.parseInt(text.substring(text.lastIndexOf(":") + 1)));
    }
}
